@@ -60,7 +60,13 @@
 
 #ifdef Q_OS_LINUX
 
-# if !defined(Q_OS_ANDROID) || !defined(Q_PROCESSOR_X86)
+# ifdef Q_OS_ANDROID
+#  include <android/api-level.h>
+# else
+#  define __ANDROID_API__ 21
+# endif
+
+# if !defined(Q_OS_ANDROID) || (!defined(Q_PROCESSOR_X86) && __ANDROID_API__ < 21)
 struct termios2 {
     tcflag_t c_iflag;       /* input mode flags */
     tcflag_t c_oflag;       /* output mode flags */
@@ -527,7 +533,8 @@ bool QSerialPortPrivate::waitForBytesWritten(int msecs)
     for (;;) {
         bool readyToRead = false;
         bool readyToWrite = false;
-        if (!waitForReadOrWrite(&readyToRead, &readyToWrite, true, !writeBuffer.isEmpty(),
+        const bool checkRead = q_func()->isReadable();
+        if (!waitForReadOrWrite(&readyToRead, &readyToWrite, checkRead, !writeBuffer.isEmpty(),
                                 qt_subtract_from_timeout(msecs, stopWatch.elapsed()))) {
             return false;
         }
@@ -767,7 +774,7 @@ bool QSerialPortPrivate::readNotification()
 
     if (readBufferMaxSize && bytesToRead > (readBufferMaxSize - buffer.size())) {
         bytesToRead = readBufferMaxSize - buffer.size();
-        if (bytesToRead == 0) {
+        if (bytesToRead <= 0) {
             // Buffer is full. User must read data from the buffer
             // before we can read more from the port.
             setReadNotificationEnabled(false);
